# Copyright (C) 2013 Linaro Limited
#
# Author: Milo Casagrande <milo.casagrande@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""Classes and functions to interact with the lava-dispatcher."""

import random
import string
import os

from lava.tool.errors import CommandError

# Default devices path, has to be joined with the dispatcher path.
DEFAULT_DEVICES_PATH = "devices"


def get_dispatcher_paths():
    """Tries to get the dispatcher paths from lava-dispatcher.

    :return A list of paths.
    """
    try:
        from lava_dispatcher.config import write_path
        return write_path()
    except ImportError:
        raise CommandError("Cannot find lava-dispatcher installation.")


def get_devices():
    """Gets the devices list from the dispatcher.

    :return A list of DeviceConfig.
    """
    try:
        from lava_dispatcher.config import get_devices
        return get_devices()
    except ImportError:
        raise CommandError("Cannot find lava-dispatcher installation.")


def get_device_file(file_name):
    """Retrieves the config file name specified, if it exists.

    :param file_name: The config file name to search.
    :return The path to the file, or None if it does not exist.
    """
    try:
        from lava_dispatcher.config import get_config_file
        return get_config_file(os.path.join(DEFAULT_DEVICES_PATH,
                                            file_name))
    except ImportError:
        raise CommandError("Cannot find lava-dispatcher installation.")


def choose_devices_path(paths):
    """Picks the first path that is writable by the user.

    :param paths: A list of paths.
    :return The first path where it is possible to write.
    """
    valid_path = None
    for path in paths:
        path = os.path.join(path, DEFAULT_DEVICES_PATH)
        if os.path.exists(path):
            name = "".join(random.choice(string.ascii_letters)
                           for x in range(6))
            test_file = os.path.join(path, name)
            try:
                fp = open(test_file, 'a')
                fp.close()
            except IOError:
                # Cannot write here.
                continue
            else:
                valid_path = path
                if os.path.isfile(test_file):
                    os.unlink(test_file)
                break
        else:
            try:
                os.makedirs(path)
            except OSError:
                # Cannot write here either.
                continue
            else:
                valid_path = path
                break
    else:
        raise CommandError("Insufficient permissions to create new "
                           "devices.")
    return valid_path


def get_devices_path():
    """Gets the path to the devices in the LAVA dispatcher."""
    return choose_devices_path(get_dispatcher_paths())



# Copyright (C) 2013 Linaro Limited
#
# Author: Milo Casagrande <milo.casagrande@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""Device class."""

import re

from copy import deepcopy

from lava.device.templates import (
    DEFAULT_TEMPLATE,
    HOSTNAME_PARAMETER,
    KNOWN_TEMPLATES,
)
from lava.helper.template import expand_template


def __re_compile(name):
    """Creates a generic regex for the specified device name.

    :param name: The name of the device.
    :return A Pattern object.
    """
    return re.compile('^.*{0}.*'.format(name), re.I)


# Dictionary of know devices.
# Keys are the general device name taken from lava.device.templates, values
# are tuples of: a regex matcher to match the device, and the device associated
# template.
KNOWN_DEVICES = dict([(device, (__re_compile(device), template))
                     for device, template in KNOWN_TEMPLATES.iteritems()])


class Device(object):

    """A generic device."""

    def __init__(self, data, hostname=None):
        self.data = deepcopy(data)
        self.hostname = hostname

    def write(self, conf_file):
        """Writes the object to file.

        :param conf_file: The full path of the file where to write."""
        with open(conf_file, 'w') as write_file:
            write_file.write(str(self))

    def update(self, config):
        """Updates the Device object values based on the provided config.

        :param config: A Config instance.
        """
        # We should always have a hostname, since it defaults to the name
        # given on the command line for the config file.
        if self.hostname is not None:
            # We do not ask the user again this parameter.
            self.data[HOSTNAME_PARAMETER.id].value = self.hostname
            self.data[HOSTNAME_PARAMETER.id].asked = True

        expand_template(self.data, config)

    def __str__(self):
        string_list = []
        for key, value in self.data.iteritems():
            string_list.append("{0} = {1}\n".format(str(key), str(value)))
        return "".join(string_list)


def get_known_device(name):
    """Tries to match a device name with a known device type.

    :param name: The name of the device we want matched to a real device.
    :return A Device instance.
        """
    instance = Device(DEFAULT_TEMPLATE, hostname=name)
    for _, (matcher, dev_template) in KNOWN_DEVICES.iteritems():
        if matcher.match(name):
            instance = Device(dev_template, hostname=name)
            break
    return instance



# Copyright (C) 2013 Linaro Limited
#
# Author: Milo Casagrande <milo.casagrande@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""Commands to run or submit a script."""

import os
import tempfile

from lava.helper.command import BaseCommand
from lava.job import DEFAULT_JOB_FILENAME
from lava.testdef import DEFAULT_TESTDEF_FILENAME
from lava.tool.command import CommandGroup
from lava_tool.utils import verify_path_non_existance


class script(CommandGroup):

    """LAVA script file handling."""

    namespace = "lava.script.commands"


class ScriptBaseCommand(BaseCommand):

    def _create_tmp_job_file(self, script_file):
        """Creates a temporary job file to run or submit the passed file.

        The temporary job file and its accessory test definition file are
        not removed by this method.

        :param script_file: The script file that has to be run or submitted.
        :return A tuple with the job file path, and the test definition path.
        """
        script_file = os.path.abspath(script_file)
        verify_path_non_existance(script_file)

        temp_dir = tempfile.gettempdir()

        # The name of the job and testdef files.
        job_file = os.path.join(temp_dir, DEFAULT_JOB_FILENAME)
        testdef_file = os.path.join(temp_dir, DEFAULT_TESTDEF_FILENAME)

        # The steps that the testdef file should have. We need to change it
        # from the default one, since the users are passing their own file.
        steps = "./" + os.path.basename(script_file)
        testdef_file = self.create_test_definition(testdef_file,
                                                   steps=steps)

        # The content of the tar file.
        tar_content = [script_file, testdef_file]
        job_file = self.create_tar_repo_job(job_file, testdef_file,
                                            tar_content)

        return (job_file, testdef_file)


class run(ScriptBaseCommand):

    """Runs the specified shell script on a local device."""

    @classmethod
    def register_arguments(cls, parser):
        super(run, cls).register_arguments(parser)
        parser.add_argument("FILE", help="Shell script file to run.")

    def invoke(self):
        job_file = ""
        testdef_file = ""

        try:
            job_file, testdef_file = self._create_tmp_job_file(self.args.FILE)
            super(run, self).run(job_file)
        finally:
            if os.path.isfile(job_file):
                os.unlink(job_file)
            if os.path.isfile(testdef_file):
                os.unlink(testdef_file)


class submit(ScriptBaseCommand):

    """Submits the specified shell script to a LAVA server."""

    @classmethod
    def register_arguments(cls, parser):
        super(submit, cls).register_arguments(parser)
        parser.add_argument("FILE", help="Shell script file to send.")

    def invoke(self):
        job_file = ""
        testdef_file = ""

        try:
            job_file, testdef_file = self._create_tmp_job_file(self.args.FILE)
            super(submit, self).submit(job_file)
        finally:
            if os.path.isfile(job_file):
                os.unlink(job_file)
            if os.path.isfile(testdef_file):
                os.unlink(testdef_file)



# Copyright (C) 2013 Linaro Limited
#
# Author: Milo Casagrande <milo.casagrande@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

import StringIO
import base64
import os
import tarfile
import tempfile
import types
import subprocess
import sys
import urlparse

from lava.tool.errors import CommandError


def has_command(command):
    """Checks that the given command is available.

    :param command: The name of the command to check availability.
    """
    command_available = True
    try:
        subprocess.check_call(["which", command],
                              stdout=open(os.path.devnull, 'w'))
    except subprocess.CalledProcessError:
        command_available = False
    return command_available


def to_list(value):
    """Return a list from the passed value.

    :param value: The parameter to turn into a list.
    """
    return_value = []
    if isinstance(value, types.StringType):
        return_value = [value]
    else:
        return_value = list(value)
    return return_value


def create_tar(paths):
    """Creates a temporary tar file with the provided paths.

    The tar file is not deleted at the end, it has to be delete by who calls
    this function.

    If just a directory is passed, it will be flattened out: its contents will
    be added, but not the directory itself.

    :param paths: List of paths to be included in the tar archive.
    :type list
    :return The path to the temporary tar file.
    """
    paths = to_list(paths)
    try:
        temp_tar_file = tempfile.NamedTemporaryFile(suffix=".tar",
                                                    delete=False)
        with tarfile.open(temp_tar_file.name, "w") as tar_file:
            for path in paths:
                full_path = os.path.abspath(path)
                if os.path.isfile(full_path):
                    arcname = os.path.basename(full_path)
                    tar_file.add(full_path, arcname=arcname)
                elif os.path.isdir(full_path):
                    # If we pass a directory, flatten it out.
                    # List its contents, and add them as they are.
                    for element in os.listdir(full_path):
                        arcname = element
                        tar_file.add(os.path.join(full_path, element),
                                     arcname=arcname)
        return temp_tar_file.name
    except tarfile.TarError:
        raise CommandError("Error creating the temporary tar archive.")


def base64_encode(path):
    """Encode in base64 the provided file.

    :param path: The path to a file.
    :return The file content encoded in base64.
    """
    if os.path.isfile(path):
        encoded_content = StringIO.StringIO()

        try:
            with open(path) as read_file:
                base64.encode(read_file, encoded_content)

            return encoded_content.getvalue().strip()
        except IOError:
            raise CommandError("Cannot read file "
                               "'{0}'.".format(path))
    else:
        raise CommandError("Provided path does not exists or is not a file: "
                           "{0}.".format(path))


def retrieve_file(path, extensions):
    """Searches for a file that has one of the supported extensions.

    The path of the first file that matches one of the supported provided
    extensions will be returned. The files are examined in alphabetical
    order.

    :param path: Where to look for the file.
    :param extensions: A list of extensions the file to look for should
                       have.
    :return The full path of the file.
    """
    if os.path.isfile(path):
        if check_valid_extension(path, extensions):
            retrieved_path = path
        else:
            raise CommandError("The provided file '{0}' is not "
                               "valid: extension not supported.".format(path))
    else:
        dir_listing = os.listdir(path)
        dir_listing.sort()

        for element in dir_listing:
            if element.startswith("."):
                continue

            element_path = os.path.join(path, element)
            if os.path.isdir(element_path):
                continue
            elif os.path.isfile(element_path):
                if check_valid_extension(element_path, extensions):
                    retrieved_path = element_path
                    break
        else:
            raise CommandError("No suitable file found in '{0}'".format(path))

    return retrieved_path


def check_valid_extension(path, extensions):
    """Checks that a file has one of the supported extensions.

    :param path: The file to check.
    :param extensions: A list of supported extensions.
    """
    is_valid = False

    local_path, file_name = os.path.split(path)
    name, full_extension = os.path.splitext(file_name)

    if full_extension:
        extension = full_extension[1:].strip().lower()
        if extension in extensions:
            is_valid = True
    return is_valid


def verify_file_extension(path, default, supported):
    """Verifies if a file has a supported extensions.

    If the file does not have one, it will add the default extension
    provided.

    :param path: The path of a file to verify.
    :param default: The default extension to use.
    :param supported: A list of supported extensions to check against.
    :return The path of the file.
    """
    full_path, file_name = os.path.split(path)
    name, extension = os.path.splitext(file_name)
    if not extension:
        path = ".".join([path, default])
    elif extension[1:].lower() not in supported:
        path = os.path.join(full_path, ".".join([name, default]))
    return path


def verify_path_existance(path):
    """Verifies if a given path exists on the file system.

    Raises a CommandError in case it exists.

    :param path: The path to verify.
    """
    if os.path.exists(path):
        raise CommandError("{0} already exists.".format(path))


def verify_path_non_existance(path):
    """Verifies if a given path does not exist on the file system.

    Raises a CommandError in case it does not exist.

    :param path: The path to verify.
    """
    if not os.path.exists(path):
        raise CommandError("{0} does not exists.".format(path))


def write_file(path, content):
    """Creates a file with the specified content.

    :param path: The path of the file to write.
    :param content: What to write in the file.
    """
    try:
        with open(path, "w") as to_write:
            to_write.write(content)
    except (OSError, IOError):
        raise CommandError("Error writing file '{0}'".format(path))


def execute(cmd_args):
    """Executes the supplied command args.

    :param cmd_args: The command, and its optional arguments, to run.
    :return The command execution return code.
    """
    cmd_args = to_list(cmd_args)
    try:
        return subprocess.check_call(cmd_args)
    except subprocess.CalledProcessError:
        raise CommandError("Error running the following command: "
                           "{0}".format(" ".join(cmd_args)))


def can_edit_file(path):
    """Checks if a file can be opend in write mode.

    :param path: The path to the file.
    :return True if it is possible to write on the file, False otherwise.
    """
    can_edit = True
    try:
        fp = open(path, "a")
        fp.close()
    except IOError:
        can_edit = False
    return can_edit


def edit_file(file_to_edit):
    """Opens the specified file with the default file editor.

    :param file_to_edit: The file to edit.
    """
    editor = os.environ.get("EDITOR", None)
    if editor is None:
        if has_command("sensible-editor"):
            editor = "sensible-editor"
        elif has_command("xdg-open"):
            editor = "xdg-open"
        else:
            # We really do not know how to open a file.
            print >> sys.stdout, ("Cannot find an editor to open the "
                                  "file '{0}'.".format(file_to_edit))
            print >> sys.stdout, ("Either set the 'EDITOR' environment "
                                  "variable, or install 'sensible-editor' "
                                  "or 'xdg-open'.")
            sys.exit(-1)
    try:
        subprocess.Popen([editor, file_to_edit]).wait()
    except Exception:
        raise CommandError("Error opening the file '{0}' with the "
                           "following editor: {1}.".format(file_to_edit,
                                                           editor))


def verify_and_create_url(endpoint):
    """Checks that the provided values make a correct URL.

    If the server address does not contain a scheme, by default it will use
    HTTPS.
    The endpoint is then added at the URL.

    :param server: A server URL to verify.
    :return A URL.
    """
    url = ""
    if endpoint:
        scheme, netloc, path, params, query, fragment = \
            urlparse.urlparse(endpoint)
        if not scheme:
            scheme = "https"
        if not netloc:
            netloc, path = path, ""

        url = urlparse.urlunparse(
            (scheme, netloc, path, params, query, fragment))

        if url[-1:] != "/":
            url += "/"

    return url


def create_dir(path, dir_name=None):
    """Checks if a directory does not exists, and creates it.

    :param path: The path where the directory should be created.
    :param dir_name: An optional name for a directory to be created at
                     path (dir_name will be joined with path).
    :return The path of the created directory."""
    created_dir = path
    if dir_name:
        created_dir = os.path.join(path, dir_name)

    if not os.path.isdir(created_dir):
        try:
            os.makedirs(created_dir)
        except OSError:
            raise CommandError("Cannot create directory "
                               "'{0}'.".format(created_dir))
    return created_dir



# Copyright (C) 2013 Linaro Limited
#
# Author: Milo Casagrande <milo.casagrande@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""
lava.config unit tests.
"""

import os
import shutil
import sys
import tempfile

from StringIO import StringIO
from mock import (
    MagicMock,
    call,
    patch,
)

from lava.config import (
    Config,
    InteractiveCache,
    InteractiveConfig,
)
from lava.helper.tests.helper_test import HelperTest
from lava.parameter import (
    Parameter,
    ListParameter,
)
from lava.tool.errors import CommandError


class ConfigTestCase(HelperTest):
    """General test case class for the different Config classes."""
    def setUp(self):
        super(ConfigTestCase, self).setUp()
        self.param1 = Parameter("foo")
        self.param2 = Parameter("bar", depends=self.param1)


class TestConfigSave(ConfigTestCase):

    """Used to test the save() method of config class.

    Done here since in the other tests we want to mock the atexit save call
    in order not to write the file, or accidentaly overwrite the real
    user file.
    """

    def setUp(self):
        super(TestConfigSave, self).setUp()
        self.config = Config()
        self.config.config_file = self.temp_file.name

    def test_config_save(self):
        self.config.put_parameter(self.param1, "foo")
        self.config.save()

        expected = "[DEFAULT]\nfoo = foo\n\n"
        obtained = ""
        with open(self.temp_file.name) as tmp_file:
            obtained = tmp_file.read()
        self.assertEqual(expected, obtained)

    def test_save_list_param(self):
        # Tests that when saved to file, the ListParameter parameter is stored
        # correctly.
        param_values = ["foo", "more than one words", "bar"]
        list_param = ListParameter("list")
        list_param.set(param_values)

        self.config.put_parameter(list_param, param_values)
        self.config.save()

        expected = "[DEFAULT]\nlist = " + ",".join(param_values) + "\n\n"
        obtained = ""
        with open(self.temp_file.name, "r") as read_file:
            obtained = read_file.read()
        self.assertEqual(expected, obtained)


class ConfigTest(ConfigTestCase):

    def setUp(self):
        super(ConfigTest, self).setUp()

        self.config_dir = os.path.join(tempfile.gettempdir(), "config")
        self.xdg_resource = os.path.join(self.config_dir, "linaro")
        self.lavatool_resource = os.path.join(self.xdg_resource, "lava-tool")

        os.makedirs(self.lavatool_resource)

        self.config = Config()
        self.config._ensure_xdg_dirs = MagicMock(
            return_value=self.lavatool_resource)
        self.config.save = MagicMock()

    def tearDown(self):
        super(ConfigTest, self).tearDown()
        if os.path.isdir(self.config_dir):
            shutil.rmtree(self.config_dir)

    def test_ensure_xdg_dirs(self):
        # Test that xdg can create the correct cache path, we remove it
        # at the end since we patch the default value.
        obtained = self.config._ensure_xdg_dirs()
        self.assertEquals(self.lavatool_resource, obtained)

    def test_config_file(self):
        expected = os.path.join(self.lavatool_resource, "lava-tool.ini")
        obtained = self.config.config_file
        self.assertEquals(expected, obtained)

    def test_config_put_in_cache_0(self):
        self.config._put_in_cache("key", "value", "section")
        self.assertEqual(self.config._cache["section"]["key"], "value")

    def test_config_get_from_cache_0(self):
        self.config._put_in_cache("key", "value", "section")
        obtained = self.config._get_from_cache(Parameter("key"), "section")
        self.assertEqual("value", obtained)

    def test_config_get_from_cache_1(self):
        self.config._put_in_cache("key", "value", "DEFAULT")
        obtained = self.config._get_from_cache(Parameter("key"), "DEFAULT")
        self.assertEqual("value", obtained)

    def test_config_put_0(self):
        # Puts a value in the DEFAULT section.
        self.config._put_in_cache = MagicMock()
        self.config.put("foo", "foo")
        expected = "foo"
        obtained = self.config._config_backend.get("DEFAULT", "foo")
        self.assertEqual(expected, obtained)

    def test_config_put_1(self):
        # Puts a value in a new section.
        self.config._put_in_cache = MagicMock()
        self.config.put("foo", "foo", "bar")
        expected = "foo"
        obtained = self.config._config_backend.get("bar", "foo")
        self.assertEqual(expected, obtained)

    def test_config_put_parameter_0(self):
        self.config._calculate_config_section = MagicMock(return_value="")
        self.assertRaises(CommandError, self.config.put_parameter, self.param1)

    @patch("lava.config.Config.put")
    def test_config_put_parameter_1(self, mocked_config_put):
        self.config._calculate_config_section = MagicMock(
            return_value="DEFAULT")

        self.param1.value = "bar"
        self.config.put_parameter(self.param1)

        self.assertEqual(mocked_config_put.mock_calls,
                         [call("foo", "bar", "DEFAULT")])

    def test_config_get_0(self):
        # Tests that with a non existing parameter, it returns None.
        param = Parameter("baz")
        self.config._get_from_cache = MagicMock(return_value=None)
        self.config._calculate_config_section = MagicMock(
            return_value="DEFAULT")

        expected = None
        obtained = self.config.get(param)
        self.assertEqual(expected, obtained)

    def test_config_get_1(self):
        self.config.put_parameter(self.param1, "foo")
        self.config._get_from_cache = MagicMock(return_value=None)
        self.config._calculate_config_section = MagicMock(
            return_value="DEFAULT")

        expected = "foo"
        obtained = self.config.get(self.param1)
        self.assertEqual(expected, obtained)

    def test_calculate_config_section_0(self):
        expected = "DEFAULT"
        obtained = self.config._calculate_config_section(self.param1)
        self.assertEqual(expected, obtained)

    def test_calculate_config_section_1(self):
        self.config.put_parameter(self.param1, "foo")
        expected = "foo=foo"
        obtained = self.config._calculate_config_section(self.param2)
        self.assertEqual(expected, obtained)

    def test_config_get_from_backend_public(self):
        # Need to to this, since we want a clean Config instance, with
        # a config_file with some content.
        with open(self.config.config_file, "w") as write_config:
            write_config.write("[DEFAULT]\nfoo=bar\n")
        param = Parameter("foo")
        obtained = self.config.get_from_backend(param)
        self.assertEquals("bar", obtained)


class InteractiveConfigTest(ConfigTestCase):

    def setUp(self):
        super(InteractiveConfigTest, self).setUp()
        self.config = InteractiveConfig()
        self.config.save = MagicMock()
        self.config.config_file = self.temp_file.name

    @patch("lava.config.Config.get", new=MagicMock(return_value=None))
    def test_non_interactive_config_0(self):
        # Try to get a value that does not exists, users just press enter when
        # asked for a value. Value will be empty.
        self.config.force_interactive = False
        sys.stdin = StringIO("\n")
        value = self.config.get(Parameter("foo"))
        self.assertEqual("", value)

    @patch("lava.config.Config.get", new=MagicMock(return_value="value"))
    def test_non_interactive_config_1(self):
        # Parent class config returns value, but we are not interactive.
        self.config.force_interactive = False
        value = self.config.get(Parameter("foo"))
        self.assertEqual("value", value)

    @patch("lava.config.Config.get", new=MagicMock(return_value=None))
    def test_non_interactive_config_2(self):
        self.config.force_interactive = False
        expected = "bar"
        sys.stdin = StringIO(expected)
        value = self.config.get(Parameter("foo"))
        self.assertEqual(expected, value)

    @patch("lava.config.Config.get", new=MagicMock(return_value="value"))
    def test_interactive_config_0(self):
        # We force to be interactive, meaning that even if a value is found,
        # it will be asked anyway.
        self.config.force_interactive = True
        expected = "a_new_value"
        sys.stdin = StringIO(expected)
        value = self.config.get(Parameter("foo"))
        self.assertEqual(expected, value)

    @patch("lava.config.Config.get", new=MagicMock(return_value="value"))
    def test_interactive_config_1(self):
        # Force to be interactive, but when asked for the new value press
        # Enter. The old value should be returned.
        self.config.force_interactive = True
        sys.stdin = StringIO("\n")
        value = self.config.get(Parameter("foo"))
        self.assertEqual("value", value)

    def test_calculate_config_section_0(self):
        self.config.force_interactive = True
        obtained = self.config._calculate_config_section(self.param1)
        expected = "DEFAULT"
        self.assertEqual(expected, obtained)

    def test_calculate_config_section_1(self):
        self.param1.set("foo")
        self.param2.depends.asked = True
        self.config.force_interactive = True
        obtained = self.config._calculate_config_section(self.param2)
        expected = "foo=foo"
        self.assertEqual(expected, obtained)

    def test_calculate_config_section_2(self):
        self.config.force_interactive = True
        self.config.config_backend.get = MagicMock(return_value=None)
        sys.stdin = StringIO("baz")
        expected = "foo=baz"
        obtained = self.config._calculate_config_section(self.param2)
        self.assertEqual(expected, obtained)

    def test_calculate_config_section_3(self):
        # Tests that when a parameter has its value in the cache and also on
        # file, we honor the cached version.
        self.param1.set("bar")
        self.param2.depends.asked = True
        self.config.force_interactive = True
        expected = "foo=bar"
        obtained = self.config._calculate_config_section(self.param2)
        self.assertEqual(expected, obtained)

    @patch("lava.config.Config.get", new=MagicMock(return_value=None))
    @patch("lava.parameter.sys.exit")
    @patch("lava.parameter.raw_input", create=True)
    def test_interactive_config_exit(self, mocked_raw, mocked_sys_exit):
        self.config._calculate_config_section = MagicMock(
            return_value="DEFAULT")

        mocked_raw.side_effect = KeyboardInterrupt()

        self.config.force_interactive = True
        self.config.get(self.param1)
        self.assertTrue(mocked_sys_exit.called)

    @patch("lava.parameter.raw_input", create=True)
    def test_interactive_config_with_list_parameter(self, mocked_raw_input):
        # Tests that we get a list back in the Config class when using
        # ListParameter and that it contains the expected values.
        expected = ["foo", "bar"]
        mocked_raw_input.side_effect = expected + ["\n"]
        obtained = self.config.get(ListParameter("list"))
        self.assertIsInstance(obtained, list)
        self.assertEqual(expected, obtained)



# Copyright (C) 2013 Linaro Limited
#
# Author: Milo Casagrande <milo.casagrande@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""
lava.device.commands unit tests.
"""

import os

from mock import (
    MagicMock,
    call,
    patch,
)

from lava.device.commands import (
    add,
    config,
    remove,
)
from lava.helper.tests.helper_test import HelperTest
from lava.tool.errors import CommandError


class AddCommandTest(HelperTest):

    def test_register_argument(self):
        # Make sure that the parser add_argument is called and we have the
        # correct argument.
        add_command = add(self.parser, self.args)
        add_command.register_arguments(self.parser)
        name, args, kwargs = self.parser.method_calls[0]
        self.assertIn("--non-interactive", args)

        name, args, kwargs = self.parser.method_calls[1]
        self.assertIn("DEVICE", args)

    @patch("lava.device.commands.edit_file", create=True)
    @patch("lava.device.Device.__str__")
    @patch("lava.device.Device.update")
    @patch("lava.device.commands.get_device_file")
    @patch("lava.device.commands.get_devices_path")
    def test_add_invoke_0(self, mocked_get_devices_path,
                          mocked_get_device_file, mocked_update, mocked_str,
                          mocked_edit_file):
        # Tests invocation of the add command. Verifies that the conf file is
        # written to disk.
        mocked_get_devices_path.return_value = self.temp_dir
        mocked_get_device_file.return_value = None
        mocked_str.return_value = ""

        add_command = add(self.parser, self.args)
        add_command.invoke()

        expected_path = os.path.join(self.temp_dir,
                                     ".".join([self.device, "conf"]))
        self.assertTrue(os.path.isfile(expected_path))

    @patch("lava.device.commands.edit_file", create=True)
    @patch("lava.device.commands.get_known_device")
    @patch("lava.device.commands.get_devices_path")
    @patch("lava.device.commands.sys.exit")
    @patch("lava.device.commands.get_device_file")
    def test_add_invoke_1(self, mocked_get_device_file, mocked_sys_exit,
                          mocked_get_devices_path, mocked_get_known_device,
                          mocked_edit_file):
        mocked_get_devices_path.return_value = self.temp_dir
        mocked_get_device_file.return_value = self.temp_file.name

        add_command = add(self.parser, self.args)
        add_command.invoke()

        self.assertTrue(mocked_sys_exit.called)


class RemoveCommandTests(HelperTest):

    def test_register_argument(self):
        # Make sure that the parser add_argument is called and we have the
        # correct argument.
        command = remove(self.parser, self.args)
        command.register_arguments(self.parser)
        name, args, kwargs = self.parser.method_calls[0]
        self.assertIn("--non-interactive", args)

        name, args, kwargs = self.parser.method_calls[1]
        self.assertIn("DEVICE", args)

    @patch("lava.device.commands.edit_file", create=True)
    @patch("lava.device.Device.__str__", return_value="")
    @patch("lava.device.Device.update")
    @patch("lava.device.commands.get_device_file")
    @patch("lava.device.commands.get_devices_path")
    def test_remove_invoke(self, get_devices_path_mock, get_device_file_mock,
                           mocked_update, mocked_str, mocked_edit_file):
        # Tests invocation of the remove command. Verifies that the conf file
        # has been correctly removed.
        # First we add a new conf file, then we remove it.
        get_device_file_mock.return_value = None
        get_devices_path_mock.return_value = self.temp_dir

        add_command = add(self.parser, self.args)
        add_command.invoke()

        expected_path = os.path.join(self.temp_dir,
                                     ".".join([self.device, "conf"]))

        # Set new values for the mocked function.
        get_device_file_mock.return_value = expected_path

        remove_command = remove(self.parser, self.args)
        remove_command.invoke()

        self.assertFalse(os.path.isfile(expected_path))

    @patch("lava.device.commands.get_device_file",
           new=MagicMock(return_value="/root"))
    def test_remove_invoke_raises(self):
        # Tests invocation of the remove command, with a non existent device
        # configuration file.
        remove_command = remove(self.parser, self.args)
        self.assertRaises(CommandError, remove_command.invoke)


class ConfigCommanTests(HelperTest):

    def test_register_argument(self):
        # Make sure that the parser add_argument is called and we have the
        # correct argument.
        command = config(self.parser, self.args)
        command.register_arguments(self.parser)
        name, args, kwargs = self.parser.method_calls[0]
        self.assertIn("--non-interactive", args)

        name, args, kwargs = self.parser.method_calls[1]
        self.assertIn("DEVICE", args)

    @patch("lava.device.commands.can_edit_file", create=True)
    @patch("lava.device.commands.edit_file", create=True)
    @patch("lava.device.commands.get_device_file")
    def test_config_invoke_0(self, mocked_get_device_file, mocked_edit_file,
                             mocked_can_edit_file):
        command = config(self.parser, self.args)

        mocked_can_edit_file.return_value = True
        mocked_get_device_file.return_value = self.temp_file.name
        command.invoke()

        self.assertTrue(mocked_edit_file.called)
        self.assertEqual([call(self.temp_file.name)],
                         mocked_edit_file.call_args_list)

    @patch("lava.device.commands.get_device_file",
           new=MagicMock(return_value=None))
    def test_config_invoke_raises_0(self):
        # Tests invocation of the config command, with a non existent device
        # configuration file.
        config_command = config(self.parser, self.args)
        self.assertRaises(CommandError, config_command.invoke)

    @patch("lava.device.commands.get_device_file",
           new=MagicMock(return_value="/etc/password"))
    def test_config_invoke_raises_1(self):
        # Tests invocation of the config command, with a non writable file.
        # Hopefully tests are not run as root.
        config_command = config(self.parser, self.args)
        self.assertRaises(CommandError, config_command.invoke)



# Copyright (C) 2013 Linaro Limited
#
# Author: Antonio Terceiro <antonio.terceiro@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""
Config class.
"""

import atexit
import os
import readline
import xdg.BaseDirectory as xdgBaseDir

from ConfigParser import (
    ConfigParser,
    NoOptionError,
    NoSectionError,
)

from lava.parameter import Parameter
from lava.tool.errors import CommandError

__all__ = ['Config', 'InteractiveCache', 'InteractiveConfig']

# Store for function calls to be made at exit time.
AT_EXIT_CALLS = set()
# Config default section.
DEFAULT_SECTION = "DEFAULT"
# This is the default base name used to create XDG resources.
DEFAULT_XDG_RESOURCE = "linaro"
# This is the default name for lava-tool resources.
DEFAULT_LAVA_TOOL_RESOURCE = "lava-tool"

HISTORY = os.path.join(os.path.expanduser("~"), ".lava_history")
try:
    readline.read_history_file(HISTORY)
except IOError:
    pass
atexit.register(readline.write_history_file, HISTORY)


def _run_at_exit():
    """Runs all the function at exit."""
    for call in list(AT_EXIT_CALLS):
        call()
atexit.register(_run_at_exit)


class Config(object):
    """A generic config object."""

    def __init__(self):
        # The cache where to store parameters.
        self._cache = {}
        self._config_file = None
        self._config_backend = None
        AT_EXIT_CALLS.add(self.save)

    @property
    def config_file(self):
        if self._config_file is None:
            self._config_file = (os.environ.get('LAVACONFIG') or
                                 os.path.join(self._ensure_xdg_dirs(),
                                              'lava-tool.ini'))
        return self._config_file

    @config_file.setter
    def config_file(self, value):
        self._config_file = value

    @property
    def config_backend(self):
        if self._config_backend is None:
            self._config_backend = ConfigParser()
            self._config_backend.read([self.config_file])
        return self._config_backend

    def _ensure_xdg_dirs(self):
        """Make sure we have the default resource.

        :return The path to the XDG resource.
        """
        return xdgBaseDir.save_config_path(DEFAULT_XDG_RESOURCE,
                                           DEFAULT_LAVA_TOOL_RESOURCE)

    def _calculate_config_section(self, parameter):
        """Calculates the config section of the specified parameter.

        :param parameter: The parameter to calculate the section of.
        :type Parameter
        :return The config section.
        """
        section = DEFAULT_SECTION
        if parameter.depends:
            section = "{0}={1}".format(parameter.depends.id,
                                       self.get(parameter.depends))
        return section

    def get(self, parameter, section=None):
        """Retrieves a Parameter value.

        The value is taken either from the Parameter itself, or from the cache,
        or from the config file.

        :param parameter: The parameter to search.
        :type Parameter
        :return The parameter value, or None if it is not found.
        """
        if not section:
            section = self._calculate_config_section(parameter)
        # Try to get the parameter value first if it has one.
        if parameter.value is not None:
            value = parameter.value
        else:
            value = self._get_from_cache(parameter, section)

        if value is None:
            value = self._get_from_backend(parameter, section)
        return value

    def get_from_backend(self, parameter, section=None):
        """Gets a configuration parameter directly from the config file."""
        if not section:
            section = self._calculate_config_section(parameter)
        return self._get_from_backend(parameter, section)

    def _get_from_backend(self, parameter, section):
        """Gets the parameter value from the config backend.

        :param parameter: The Parameter to look up.
        :param section: The section in the Config.
        """
        value = None
        try:
            value = self.config_backend.get(section, parameter.id)
        except (NoOptionError, NoSectionError):
            # Ignore, we return None.
            pass
        return value

    def _get_from_cache(self, parameter, section):
        """Looks for the specified parameter in the internal cache.

        :param parameter: The parameter to search.
        :type Parameter
        :return The parameter value, of None if it is not found.
        """
        value = None
        if section in self._cache.keys():
            if parameter.id in self._cache[section].keys():
                value = self._cache[section][parameter.id]
        return value

    def _put_in_cache(self, key, value, section=DEFAULT_SECTION):
        """Insert the passed parameter in the internal cache.

        :param parameter: The parameter to insert.
        :type Parameter
        :param section: The name of the section in the config file.
        :type str
        """
        if section not in self._cache.keys():
            self._cache[section] = {}
        self._cache[section][key] = value

    def put(self, key, value, section=DEFAULT_SECTION):
        """Adds a parameter to the config file.

        :param key: The key to add.
        :param value: The value to add.
        :param section: The name of the section as in the config file.
        """
        if (not self.config_backend.has_section(section) and
                section != DEFAULT_SECTION):
            self.config_backend.add_section(section)

        # This is done to serialize a list when ConfigParser is written to
        # file. Since there is no real support for list in ConfigParser, we
        # serialized it in a common way that can get easily deserialized.
        if isinstance(value, list):
            value = Parameter.serialize(value)

        self.config_backend.set(section, key, value)
        # Store in the cache too.
        self._put_in_cache(key, value, section)

    def put_parameter(self, parameter, value=None, section=None):
        """Adds a Parameter to the config file and cache.

        :param Parameter: The parameter to add.
        :type Parameter
        :param value: The value of the parameter. Defaults to None.
        :param section: The section where this parameter should be stored.
                        Defaults to None.
        """
        if not section:
            section = self._calculate_config_section(parameter)

        if value is None and parameter.value is not None:
            value = parameter.value
        elif value is None:
            raise CommandError("No value assigned to '{0}'.".format(
                parameter.id))
        self.put(parameter.id, value, section)

    def save(self):
        """Saves the config to file."""
        # Since we lazy load the config_backend property, this check is needed
        # when a user enters a wrong command or it will overwrite the 'config'
        # file with empty contents.
        if self._config_backend:
            with open(self.config_file, "w") as write_file:
                self.config_backend.write(write_file)


class InteractiveConfig(Config):
    """An interactive config.

    If a value is not found in the config file, it will ask it and then stores
    it.
    """
    def __init__(self, force_interactive=True):
        super(InteractiveConfig, self).__init__()
        self._force_interactive = force_interactive

    @property
    def force_interactive(self):
        return self._force_interactive

    @force_interactive.setter
    def force_interactive(self, value):
        self._force_interactive = value

    def get(self, parameter, section=None):
        """Overrides the parent one.

        The only difference with the parent one, is that it will ask to type
        a parameter value in case it is not found.
        """
        if not section:
            section = self._calculate_config_section(parameter)
        value = super(InteractiveConfig, self).get(parameter, section)

        if value is None or self.force_interactive:
            value = parameter.prompt(old_value=value)

        if value is not None and parameter.store:
            self.put(parameter.id, value, section)
        return value


class InteractiveCache(InteractiveConfig):

    """An interactive cache where parameters that can change are stored.

    This class is basically the same as the Confing and InteractiveConfig ones,
    only the base directory where the cache file is stored is different.

    In this case it will use the $XDG_CACHE_HOME value as defined in XDG.
    """

    @property
    def config_file(self):
        if self._config_file is None:
            self._config_file = (os.environ.get('LAVACACHE') or
                                 os.path.join(self._ensure_xdg_dirs(),
                                              'parameters.ini'))
        return self._config_file

    @config_file.setter
    def config_file(self, value):
        self._config_file = value

    def _ensure_xdg_dirs(self):
        """Make sure we have the default resource.

        :return The path to the XDG resource.
        """
        return xdgBaseDir.save_cache_path(DEFAULT_XDG_RESOURCE,
                                          DEFAULT_LAVA_TOOL_RESOURCE)



# Copyright (C) 2013 Linaro Limited
#
# Author: Milo Casagrande <milo.casagrande@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""
Device class unit tests.
"""

from mock import patch

from lava.config import Config
from lava.device import (
    Device,
    get_known_device,
)
from lava.device.templates import (
    HOSTNAME_PARAMETER,
    PANDA_DEVICE_TYPE,
    PANDA_CONNECTION_COMMAND,
)
from lava.helper.tests.helper_test import HelperTest
from lava.parameter import Parameter


class DeviceTest(HelperTest):

    def test_get_known_device_panda_0(self):
        # User creates a new device with a guessable name for a device.
        instance = get_known_device('panda_new_01')
        self.assertIsInstance(instance, Device)
        self.assertEqual(instance.data['device_type'].value, 'panda')

    def test_get_known_device_panda_1(self):
        # User creates a new device with a guessable name for a device.
        # Name passed has capital letters.
        instance = get_known_device('new_PanDa_02')
        self.assertIsInstance(instance, Device)
        self.assertEqual(instance.data['device_type'].value, 'panda')

    def test_get_known_device_vexpress_0(self):
        # User creates a new device with a guessable name for a device.
        # Name passed has capital letters.
        instance = get_known_device('a_VexPress_Device')
        self.assertIsInstance(instance, Device)
        self.assertEqual(instance.data['device_type'].value, 'vexpress')

    def test_get_known_device_vexpress_1(self):
        # User creates a new device with a guessable name for a device.
        instance = get_known_device('another-vexpress')
        self.assertIsInstance(instance, Device)
        self.assertIsInstance(instance.data['device_type'], Parameter)
        self.assertEqual(instance.data['device_type'].value, 'vexpress')

    @patch("lava.config.Config.save")
    def test_device_update_1(self, patched_save):
        # Tests that when calling update() on a Device, the template gets
        # updated with the correct values from a Config instance.
        hostname = "panda_device"

        config = Config()
        config._config_file = self.temp_file.name
        config.put_parameter(HOSTNAME_PARAMETER, hostname)
        config.put_parameter(PANDA_DEVICE_TYPE, "panda")
        config.put_parameter(PANDA_CONNECTION_COMMAND, "test")

        expected = {
            "hostname": hostname,
            "device_type": "panda",
            "connection_command": "test"
        }

        instance = get_known_device(hostname)
        instance.update(config)

        self.assertEqual(expected, instance.data)

    @patch("lava.config.Config.save")
    def test_device_write(self, mocked_save):
        # User tries to create a new panda device. The conf file is written
        # and contains the expected results.
        hostname = "panda_device"

        config = Config()
        config._config_file = self.temp_file.name
        config.put_parameter(HOSTNAME_PARAMETER, hostname)
        config.put_parameter(PANDA_DEVICE_TYPE, "panda")
        config.put_parameter(PANDA_CONNECTION_COMMAND, "test")

        expected = {
            "hostname": hostname,
            "device_type": "panda",
            "connection_command": "test"
        }

        instance = get_known_device(hostname)
        instance.update(config)
        instance.write(self.temp_file.name)

        expected = ("hostname = panda_device\nconnection_command = test\n"
                    "device_type = panda\n")

        obtained = ""
        with open(self.temp_file.name) as f:
            obtained = f.read()
        self.assertEqual(expected, obtained)



# Copyright (C) 2013 Linaro Limited
#
# Author: Milo Casagrande <milo.casagrande@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

""" """

import copy
from unittest import TestCase

from lava.helper.template import (
    get_key,
    set_value
)


TEST_TEMPLATE = {
    "key1": "value1",
    "key2": [
        "value2", "value3"
    ],
    "key3": [
        {
            "key4": "value4",
            "key5": "value5"
        },
        {
            "key6": "value6",
            "key7": "value7"
        },
        [
            {
                "key8": "value8"
            }
        ]
    ],
    "key10": {
        "key11": "value11"
    }
}


class TestParameter(TestCase):

    def test_get_key_simple_key(self):
        expected = "value1"
        obtained = get_key(TEST_TEMPLATE, "key1")
        self.assertEquals(expected, obtained)

    def test_get_key_nested_key(self):
        expected = "value4"
        obtained = get_key(TEST_TEMPLATE, "key4")
        self.assertEquals(expected, obtained)

    def test_get_key_nested_key_1(self):
        expected = "value7"
        obtained = get_key(TEST_TEMPLATE, "key7")
        self.assertEquals(expected, obtained)

    def test_get_key_nested_key_2(self):
        expected = "value8"
        obtained = get_key(TEST_TEMPLATE, "key8")
        self.assertEquals(expected, obtained)

    def test_get_key_nested_key_3(self):
        expected = "value11"
        obtained = get_key(TEST_TEMPLATE, "key11")
        self.assertEquals(expected, obtained)

    def test_set_value_0(self):
        data = copy.deepcopy(TEST_TEMPLATE)
        expected = "foo"
        set_value(data, "key1", expected)
        obtained = get_key(data, "key1")
        self.assertEquals(expected, obtained)

    def test_set_value_1(self):
        data = copy.deepcopy(TEST_TEMPLATE)
        expected = "foo"
        set_value(data, "key6", expected)
        obtained = get_key(data, "key6")
        self.assertEquals(expected, obtained)

    def test_set_value_2(self):
        data = copy.deepcopy(TEST_TEMPLATE)
        expected = "foo"
        set_value(data, "key11", expected)
        obtained = get_key(data, "key11")
        self.assertEquals(expected, obtained)



# Copyright (C) 2010, 2011 Linaro Limited
#
# Author: Michael Hudson-Doyle <michael.hudson@linaro.org>
#
# This file is part of lava-scheduler-tool.
#
# lava-scheduler-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-scheduler-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-scheduler-tool.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import argparse
import xmlrpclib

from lava_tool.authtoken import AuthenticatingServerProxy, KeyringAuthBackend
from lava.tool.command import Command, CommandGroup
from lava.tool.errors import CommandError


class scheduler(CommandGroup):
    """
    Interact with LAVA Scheduler
    """

    namespace = "lava.scheduler.commands"


class submit_job(Command):
    """
    Submit a job to lava-scheduler
    """

    @classmethod
    def register_arguments(cls, parser):
        super(submit_job, cls).register_arguments(parser)
        parser.add_argument("SERVER")
        parser.add_argument("JSON_FILE")

    def invoke(self):
        server = AuthenticatingServerProxy(
            self.args.SERVER, auth_backend=KeyringAuthBackend())
        with open(self.args.JSON_FILE, 'rb') as stream:
            command_text = stream.read()
        try:
            job_id = server.scheduler.submit_job(command_text)
        except xmlrpclib.Fault, e:
            raise CommandError(str(e))
        else:
            print "submitted as job id:", job_id


class resubmit_job(Command):

    @classmethod
    def register_arguments(self, parser):
        parser.add_argument("SERVER")
        parser.add_argument("JOB_ID")

    def invoke(self):
        server = AuthenticatingServerProxy(
            self.args.SERVER, auth_backend=KeyringAuthBackend())
        try:
            job_id = server.scheduler.resubmit_job(self.args.JOB_ID)
        except xmlrpclib.Fault, e:
            raise CommandError(str(e))
        else:
            print "resubmitted as job id:", job_id


class cancel_job(Command):

    @classmethod
    def register_arguments(self, parser):
        parser.add_argument("SERVER")
        parser.add_argument("JOB_ID")

    def invoke(self):
        server = AuthenticatingServerProxy(
            self.args.SERVER, auth_backend=KeyringAuthBackend())
        server.scheduler.cancel_job(self.args.JOB_ID)


class job_output(Command):
    """
    Get job output from the scheduler.
    """

    @classmethod
    def register_arguments(cls, parser):
        super(job_output, cls).register_arguments(parser)
        parser.add_argument("SERVER")
        parser.add_argument("JOB_ID",
                            help="Job ID to download output file")
        parser.add_argument("--overwrite",
                            action="store_true",
                            help="Overwrite files on the local disk")
        parser.add_argument("--output", "-o",
                            type=argparse.FileType("wb"),
                            default=None,
                            help="Alternate name of the output file")

    def invoke(self):
        if self.args.output is None:
            filename = str(self.args.JOB_ID) + '_output.txt'
            if os.path.exists(filename) and not self.args.overwrite:
                print >> sys.stderr, "File {filename!r} already exists".format(
                    filename=filename)
                print >> sys.stderr, "You may pass --overwrite to write over it"
                return -1
            stream = open(filename, "wb")
        else:
            stream = self.args.output
            filename = self.args.output.name

        server = AuthenticatingServerProxy(
            self.args.SERVER, auth_backend=KeyringAuthBackend())
        stream.write(server.scheduler.job_output(self.args.JOB_ID).data)

        print "Downloaded job output of {0} to file {1!r}".format(
            self.args.JOB_ID, filename)


class job_status(Command):
    """
    Get job status and bundle sha1, if it existed, from the scheduler.
    """

    @classmethod
    def register_arguments(cls, parser):
        super(job_status, cls).register_arguments(parser)
        parser.add_argument("SERVER")
        parser.add_argument("JOB_ID",
                            help="Job ID to check the status")

    def invoke(self):
        server = AuthenticatingServerProxy(
            self.args.SERVER, auth_backend=KeyringAuthBackend())
        job_status = server.scheduler.job_status(self.args.JOB_ID)

        print "Job ID: %s\nJob Status: %s\nBundle SHA1: %s" % \
            (str(self.args.JOB_ID), job_status['job_status'],
             job_status['bundle_sha1'])



# Copyright (C) 2013 Linaro Limited
#
# Author: Milo Casagrande <milo.casagrande@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""lava.helper.command module tests."""

from mock import MagicMock, patch


from lava.helper.command import BaseCommand
from lava.helper.tests.helper_test import HelperTest


class BaseCommandTests(HelperTest):

    def test_register_argument(self):
        # Make sure that the parser add_argument is called and we have the
        # correct argument.
        command = BaseCommand(self.parser, self.args)
        command.register_arguments(self.parser)
        name, args, kwargs = self.parser.method_calls[0]
        self.assertIn("--non-interactive", args)

    @patch("lava.helper.command.AuthenticatingServerProxy", create=True)
    def test_authenticated_server(self, mocked_auth_server):
        command = BaseCommand(self.parser, self.args)
        command.config = MagicMock()
        command.config.get = MagicMock()
        command.config.get.side_effect = ["www.example.org", "RPC"]

        command.authenticated_server()

        self.assertTrue(mocked_auth_server.called)



# Copyright (C) 2013 Linaro Limited
#
# Author: Antonio Terceiro <antonio.terceiro@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""
Unit tests for the Job class
"""

import os
import json
import tempfile

from mock import patch

from lava.config import Config
from lava.helper.tests.helper_test import HelperTest
from lava.job import Job
from lava.job.templates import BOOT_TEST
from lava.parameter import Parameter


class JobTest(HelperTest):

    @patch("lava.config.Config.save")
    def setUp(self, mocked_config):
        super(JobTest, self).setUp()
        self.config = Config()
        self.config.config_file = self.temp_file.name

    def test_from_template(self):
        template = {}
        job = Job(template, self.temp_file.name)
        self.assertEqual(job.data, template)
        self.assertIsNot(job.data, template)

    def test_update_data(self):
        image = "/path/to/panda.img"
        param1 = Parameter("device_type")
        param2 = Parameter("image", depends=param1)
        self.config.put_parameter(param1, "panda")
        self.config.put_parameter(param2, image)

        job = Job(BOOT_TEST, self.temp_file.name)
        job.update(self.config)

        self.assertEqual(job.data['device_type'], "panda")
        self.assertEqual(job.data['actions'][0]["parameters"]["image"], image)

    def test_write(self):
        try:
            orig_data = {"foo": "bar"}
            job_file = os.path.join(tempfile.gettempdir(), "a_json_file.json")
            job = Job(orig_data, job_file)
            job.write()

            output = ""
            with open(job_file) as read_file:
                output = read_file.read()

            data = json.loads(output)
            self.assertEqual(data, orig_data)
        finally:
            os.unlink(job_file)

    def test_writes_nicely_formatted_json(self):
        try:
            orig_data = {"foo": "bar"}
            job_file = os.path.join(tempfile.gettempdir(), "b_json_file.json")
            job = Job(orig_data, job_file)
            job.write()

            output = ""
            with open(job_file) as read_file:
                output = read_file.read()

            self.assertTrue(output.startswith("{\n"))
        finally:
            os.unlink(job_file)



# Copyright (C) 2010, 2011, 2012 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""
lava.tool.errors
================

Error classes for LAVA Tool.
"""


class CommandError(Exception):

    """
    Raise this from a Command's invoke() method to display an error nicely.

    lava-tool will exit with a status of 1 if this is raised.
    """



# Copyright (C) 2013 Linaro Limited
#
# Author: Milo Casagrande <milo.casagrande@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""
Unittests for the ShellScript class.
"""

import os
import stat

from lava.helper.tests.helper_test import HelperTest
from lava.script import ShellScript


class ShellScriptTests(HelperTest):

    """ShellScript tests."""

    def test_create_file(self):
        # Tests that a shell script is actually written.
        try:
            temp_file = self.tmp("a_shell_test")
            script = ShellScript(temp_file)
            script.write()

            self.assertTrue(os.path.isfile(temp_file))
        finally:
            os.unlink(temp_file)

    def test_assure_executable(self):
        # Tests that the shell script created is executable.
        try:
            temp_file = self.tmp("a_shell_test")
            script = ShellScript(temp_file)
            script.write()

            expected = (stat.S_IRWXU | stat.S_IRWXG | stat.S_IROTH |
                        stat.S_IXOTH)

            obtained = stat.S_IMODE(os.stat(temp_file).st_mode)
            self.assertEquals(expected, obtained)
        finally:
            os.unlink(temp_file)

    def test_shell_script_content(self):
        # Tests that the shell script created contains the exepcted content.
        try:
            temp_file = self.tmp("a_shell_test")
            script = ShellScript(temp_file)
            script.write()

            obtained = ""
            with open(temp_file) as read_file:
                obtained = read_file.read()

            expected = ("#!/bin/sh\n# Automatic generated "
                        "content by lava-tool.\n# Please add your own "
                        "instructions.\n#\n# You can use all the avialable "
                        "Bash commands.\n#\n# For the available LAVA "
                        "commands, see:\n#    http://lava.readthedocs.org/\n"
                        "#\n")

            self.assertEquals(expected, obtained)
        finally:
            os.unlink(temp_file)



# Copyright (C) 2013 Linaro Limited
#
# Author: Antonio Terceiro <antonio.terceiro@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""
LAVA job commands.
"""

import os

from lava.helper.command import BaseCommand
from lava.job import Job
from lava.job.templates import (
    BOOT_TEST_KEY,
    JOB_TYPES,
)
from lava.tool.command import CommandGroup
from lava.tool.errors import CommandError


class job(CommandGroup):
    """LAVA job file handling."""
    namespace = 'lava.job.commands'


class new(BaseCommand):
    """Creates a new job file."""

    @classmethod
    def register_arguments(cls, parser):
        super(new, cls).register_arguments(parser)
        parser.add_argument("FILE", help=("Job file to be created."))
        parser.add_argument("--type",
                            help=("The type of job to create. Defaults to "
                                  "'{0}'.".format(BOOT_TEST_KEY)),
                            choices=JOB_TYPES.keys(),
                            default=BOOT_TEST_KEY)

    def invoke(self, job_template=None):
        if not job_template:
            job_template = JOB_TYPES.get(self.args.type)

        full_path = os.path.abspath(self.args.FILE)

        job_instance = Job(job_template, full_path)
        job_instance.update(self.config)
        job_instance.write()


class submit(BaseCommand):

    """Submits the specified job file."""

    @classmethod
    def register_arguments(cls, parser):
        super(submit, cls).register_arguments(parser)
        parser.add_argument("FILE", help=("The job file to submit."))

    def invoke(self):
        super(submit, self).submit(self.args.FILE)


class run(BaseCommand):

    """Runs the specified job file on the local dispatcher."""

    @classmethod
    def register_arguments(cls, parser):
        super(run, cls).register_arguments(parser)
        parser.add_argument("FILE", help=("The job file to submit."))

    def invoke(self):
        super(run, self).run(self.args.FILE)


class status(BaseCommand):

    """Retrieves the status of a job."""

    @classmethod
    def register_arguments(cls, parser):
        super(status, cls).register_arguments(parser)
        parser.add_argument("JOB_ID",
                            help=("Prints status information about the "
                                  "provided job id."),
                            nargs="?",
                            default=None)

    def invoke(self):
        if self.args.JOB_ID:
            super(status, self).status(self.args.JOB_ID)
        else:
            raise CommandError("It is necessary to specify a job id.")



# Copyright (C) 2013 Linaro Limited
#
# Author: Milo Casagrande <milo.casagrande@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""Test definition templates."""

from lava.parameter import (
    Parameter,
)

DEFAULT_TESTDEF_VERSION = "1.0"
DEFAULT_TESTDEF_FORMAT = "Lava-Test Test Definition 1.0"
DEFAULT_ENVIRONMET_VALUE = "lava_test_shell"

# All these parameters will not be stored on the local config file.
NAME_PARAMETER = Parameter("name")
NAME_PARAMETER.store = False

DESCRIPTION_PARAMETER = Parameter("description", depends=NAME_PARAMETER)
DESCRIPTION_PARAMETER.store = False

TESTDEF_STEPS_KEY = "steps"

TESTDEF_TEMPLATE = {
    "metadata": {
        "name": NAME_PARAMETER,
        "format": DEFAULT_TESTDEF_FORMAT,
        "version": DEFAULT_TESTDEF_VERSION,
        "description": DESCRIPTION_PARAMETER,
        "environment": [DEFAULT_ENVIRONMET_VALUE],
    },
    "run": {
        TESTDEF_STEPS_KEY: ["./mytest.sh"]
    },
    "parse": {
        "pattern": r'^\s*(?P<test_case_id>\w+)=(?P<result>\w+)\s*$'
    }
}



# Copyright (C) 2013 Linaro Limited
#
# Author: Antonio Terceiro <antonio.terceiro@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""Job class."""

import json

from copy import deepcopy

from lava.helper.template import (
    expand_template,
    set_value,
)
from lava_tool.utils import (
    verify_file_extension,
    verify_path_existance,
    write_file
)

# A default name for job files.
DEFAULT_JOB_FILENAME = "lava-tool-job.json"
# Default job file extension.
DEFAULT_JOB_EXTENSION = "json"
# Possible extension for a job file.
JOB_FILE_EXTENSIONS = [DEFAULT_JOB_EXTENSION]


class Job(object):

    """A Job object.

    This class should be used to create new job files. The initialization
    enforces a default file name extension, and makes sure that the file is
    not already present on the file system.
    """

    def __init__(self, data, file_name):
        self.file_name = verify_file_extension(file_name,
                                               DEFAULT_JOB_EXTENSION,
                                               JOB_FILE_EXTENSIONS)
        verify_path_existance(self.file_name)
        self.data = deepcopy(data)

    def set(self, key, value):
        """Set key to the specified value.

        :param key: The key to look in the object data.
        :param value: The value to set.
        """
        set_value(self.data, key, value)

    def update(self, config):
        """Updates the Job object based on the provided config."""
        expand_template(self.data, config)

    def write(self):
        """Writes the Job object to file."""
        write_file(self.file_name, json.dumps(self.data, indent=4))



# Copyright (C) 2013 Linaro Limited
#
# Author: Milo Casagrande <milo.casagrande@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""
A test helper class.

Here we define a general test class and its own setUp and tearDown methods that
all other test classes can inherit from.
"""

import os
import shutil
import sys
import tempfile

from unittest import TestCase
from mock import (
    MagicMock,
    patch
)


class HelperTest(TestCase):
    """Helper test class that all tests under the lava package can inherit."""

    def setUp(self):
        # Need to patch it here, not as a decorator, or running the tests
        # via `./setup.py test` will fail.
        self.at_exit_patcher = patch("lava.config.AT_EXIT_CALLS", spec=set)
        self.at_exit_patcher.start()
        self.original_stdout = sys.stdout
        sys.stdout = open("/dev/null", "w")
        self.original_stderr = sys.stderr
        sys.stderr = open("/dev/null", "w")
        self.original_stdin = sys.stdin

        self.device = "a_fake_panda02"

        self.temp_file = tempfile.NamedTemporaryFile(delete=False)
        self.temp_dir = tempfile.mkdtemp()
        self.parser = MagicMock()
        self.args = MagicMock()
        self.args.interactive = MagicMock(return_value=False)
        self.args.DEVICE = self.device

    def tearDown(self):
        self.at_exit_patcher.stop()
        sys.stdin = self.original_stdin
        sys.stdout = self.original_stdout
        sys.stderr = self.original_stderr
        shutil.rmtree(self.temp_dir)
        os.unlink(self.temp_file.name)

    def tmp(self, name):
        """
        Returns the full path to a file, or directory, called `name` in a
        temporary directory.

        This method does not create the file, it only gives a full filename
        where you can actually write some data. The file will not be removed
        by this method.

        :param name: The name the file/directory should have.
        :return A path.
        """
        return os.path.join(tempfile.gettempdir(), name)



# Copyright (C) 2013 Linaro Limited
#
# Author: Milo Casagrande <milo.casagrande@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""
This is just a place where to store a template like dictionary that
will be used to serialize a Device object.
"""

from copy import copy

from lava.parameter import Parameter

# The hostname parameter is always in the DEFAULT config section.
HOSTNAME_PARAMETER = Parameter("hostname")
DEVICE_TYPE_PARAMETER = Parameter("device_type", depends=HOSTNAME_PARAMETER)
CONNECTION_COMMAND_PARMAETER = Parameter("connection_command",
                                         depends=DEVICE_TYPE_PARAMETER)

DEFAULT_TEMPLATE = {
    'hostname': HOSTNAME_PARAMETER,
    'device_type': DEVICE_TYPE_PARAMETER,
    'connection_command': CONNECTION_COMMAND_PARMAETER,
}

# Specialized copies of the parameters.
# We need this or we might end up asking the user twice the same parameter due
# to different object references when one Parameter depends on a "specialized"
# one, different from the defaults.
PANDA_DEVICE_TYPE = copy(DEVICE_TYPE_PARAMETER)
PANDA_DEVICE_TYPE.value = "panda"
PANDA_DEVICE_TYPE.asked = True

PANDA_CONNECTION_COMMAND = copy(CONNECTION_COMMAND_PARMAETER)
PANDA_CONNECTION_COMMAND.depends = PANDA_DEVICE_TYPE

VEXPRESS_DEVICE_TYPE = copy(DEVICE_TYPE_PARAMETER)
VEXPRESS_DEVICE_TYPE.value = "vexpress"
VEXPRESS_DEVICE_TYPE.asked = True

VEXPRESS_CONNECTION_COMMAND = copy(CONNECTION_COMMAND_PARMAETER)
VEXPRESS_CONNECTION_COMMAND.depends = VEXPRESS_DEVICE_TYPE

QEMU_DEVICE_TYPE = copy(DEVICE_TYPE_PARAMETER)
QEMU_DEVICE_TYPE.value = "qemu"
QEMU_DEVICE_TYPE.asked = True

QEMU_CONNECTION_COMMAND = copy(CONNECTION_COMMAND_PARMAETER)
QEMU_CONNECTION_COMMAND.depends = QEMU_DEVICE_TYPE

# Dictionary with templates of known devices.
KNOWN_TEMPLATES = {
    'panda': {
        'hostname': HOSTNAME_PARAMETER,
        'device_type': PANDA_DEVICE_TYPE,
        'connection_command': PANDA_CONNECTION_COMMAND,
    },
    'vexpress': {
        'hostname': HOSTNAME_PARAMETER,
        'device_type': VEXPRESS_DEVICE_TYPE,
        'connection_command': VEXPRESS_CONNECTION_COMMAND,
    },
    'qemu': {
        'hostname': HOSTNAME_PARAMETER,
        'device_type': QEMU_DEVICE_TYPE,
        'connection_command': QEMU_CONNECTION_COMMAND,
    }
}



# Copyright (C) 2013 Linaro Limited
#
# Author: Antonio Terceiro <antonio.terceiro@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""
Unit tests for the commands classes
"""

import json
import os

from mock import patch

from lava.config import Config
from lava.helper.tests.helper_test import HelperTest
from lava.job.commands import (
    new,
    run,
    submit,
    status,
)
from lava.parameter import Parameter
from lava.tool.errors import CommandError


class CommandTest(HelperTest):

    def setUp(self):
        super(CommandTest, self).setUp()
        self.args.FILE = self.temp_file.name
        self.args.type = "boot-test"

        self.device_type = Parameter('device_type')
        self.prebuilt_image = Parameter('prebuilt_image',
                                        depends=self.device_type)
        self.config = Config()
        self.config.put_parameter(self.device_type, 'foo')
        self.config.put_parameter(self.prebuilt_image, 'bar')


class JobNewTest(CommandTest):

    def setUp(self):
        super(JobNewTest, self).setUp()
        self.args.FILE = self.tmp("new_file.json")
        self.new_command = new(self.parser, self.args)
        self.new_command.config = self.config

    def tearDown(self):
        super(JobNewTest, self).tearDown()
        if os.path.exists(self.args.FILE):
            os.unlink(self.args.FILE)

    def test_register_arguments(self):
        new_cmd = new(self.parser, self.args)
        new_cmd.register_arguments(self.parser)

        # Make sure we do not forget about this test.
        self.assertEqual(3, len(self.parser.method_calls))

        _, args, _ = self.parser.method_calls[0]
        self.assertIn("--non-interactive", args)

        _, args, _ = self.parser.method_calls[1]
        self.assertIn("FILE", args)

        _, args, _ = self.parser.method_calls[2]
        self.assertIn("--type", args)

    def test_create_new_file(self):
        self.new_command.invoke()
        self.assertTrue(os.path.exists(self.args.FILE))

    def test_fills_in_template_parameters(self):
        self.new_command.invoke()

        data = json.loads(open(self.args.FILE).read())
        self.assertEqual(data['device_type'], 'foo')

    def test_wont_overwrite_existing_file(self):
        with open(self.args.FILE, 'w') as f:
            f.write("CONTENTS")

        self.assertRaises(CommandError, self.new_command.invoke)
        self.assertEqual("CONTENTS", open(self.args.FILE).read())


class JobSubmitTest(CommandTest):

    def test_receives_job_file_in_cmdline(self):
        command = submit(self.parser, self.args)
        command.register_arguments(self.parser)
        name, args, kwargs = self.parser.method_calls[1]
        self.assertIn("FILE", args)


class JobRunTest(CommandTest):

    def test_register_arguments(self):
        run_cmd = run(self.parser, self.args)
        run_cmd.register_arguments(self.parser)

        # Make sure we do not forget about this test.
        self.assertEqual(2, len(self.parser.method_calls))

        _, args, _ = self.parser.method_calls[0]
        self.assertIn("--non-interactive", args)

        _, args, _ = self.parser.method_calls[1]
        self.assertIn("FILE", args)

    def test_invoke_raises_0(self):
        # Users passes a non existing job file to the run command.
        self.args.FILE = self.tmp("test_invoke_raises_0.json")
        command = run(self.parser, self.args)
        self.assertRaises(CommandError, command.invoke)

    @patch("lava.helper.command.has_command", create=True)
    def test_invoke_raises_1(self, mocked_has_command):
        # User passes a valid file to the run command, but she does not have
        # the dispatcher installed.
        mocked_has_command.return_value = False
        command = run(self.parser, self.args)
        self.assertRaises(CommandError, command.invoke)


class TestsStatusCommand(CommandTest):

    def test_register_arguments(self):
        self.args.JOB_ID = "1"
        status_cmd = status(self.parser, self.args)
        status_cmd.register_arguments(self.parser)

        # Make sure we do not forget about this test.
        self.assertEqual(2, len(self.parser.method_calls))

        _, args, _ = self.parser.method_calls[0]
        self.assertIn("--non-interactive", args)

        _, args, _ = self.parser.method_calls[1]
        self.assertIn("JOB_ID", args)



# Copyright (C) 2013 Linaro Limited
#
# Author: Milo Casagrande <milo.casagrande@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""
Device specific commands class.
"""

import os
import sys

from lava.device import get_known_device
from lava.helper.command import (
    BaseCommand,
)
from lava.helper.dispatcher import (
    get_device_file,
    get_devices_path,
)
from lava.tool.command import CommandGroup
from lava.tool.errors import CommandError
from lava_tool.utils import (
    can_edit_file,
    edit_file,
)

DEVICE_FILE_SUFFIX = "conf"


class device(CommandGroup):
    """LAVA devices handling."""

    namespace = "lava.device.commands"


class add(BaseCommand):
    """Adds a new device."""

    @classmethod
    def register_arguments(cls, parser):
        super(add, cls).register_arguments(parser)
        parser.add_argument("DEVICE", help="The name of the device to add.")

    def invoke(self):
        real_file_name = ".".join([self.args.DEVICE, DEVICE_FILE_SUFFIX])

        if get_device_file(real_file_name) is not None:
            print >> sys.stdout, ("A device configuration file named '{0}' "
                                  "already exists.".format(real_file_name))
            print >> sys.stdout, ("Use 'lava device config {0}' to edit "
                                  "it.".format(self.args.DEVICE))
            sys.exit(-1)

        devices_path = get_devices_path()
        device_conf_file = os.path.abspath(os.path.join(devices_path,
                                                        real_file_name))

        device = get_known_device(self.args.DEVICE)
        device.update(self.config)
        device.write(device_conf_file)

        print >> sys.stdout, ("Created device file '{0}' in: {1}".format(
            real_file_name, devices_path))
        edit_file(device_conf_file)


class remove(BaseCommand):
    """Removes the specified device."""

    @classmethod
    def register_arguments(cls, parser):
        super(remove, cls).register_arguments(parser)
        parser.add_argument("DEVICE",
                            help="The name of the device to remove.")

    def invoke(self):
        real_file_name = ".".join([self.args.DEVICE, DEVICE_FILE_SUFFIX])
        device_conf = get_device_file(real_file_name)

        if device_conf:
            try:
                os.remove(device_conf)
                print >> sys.stdout, ("Device configuration file '{0}' "
                                      "removed.".format(real_file_name))
            except OSError:
                raise CommandError("Cannot remove file '{0}' at: {1}.".format(
                    real_file_name, os.path.dirname(device_conf)))
        else:
            print >> sys.stdout, ("No device configuration file '{0}' "
                                  "found.".format(real_file_name))


class config(BaseCommand):
    """Opens the specified device config file."""
    @classmethod
    def register_arguments(cls, parser):
        super(config, cls).register_arguments(parser)
        parser.add_argument("DEVICE",
                            help="The name of the device to edit.")

    def invoke(self):
        real_file_name = ".".join([self.args.DEVICE, DEVICE_FILE_SUFFIX])
        device_conf = get_device_file(real_file_name)

        if device_conf and can_edit_file(device_conf):
            edit_file(device_conf)
        else:
            raise CommandError("Cannot edit file '{0}'".format(real_file_name))



# Copyright (C) 2013 Linaro Limited
#
# Author: Antonio Terceiro <antonio.terceiro@linaro.org>
#
# This file is part of lava-tool.
#
# lava-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-tool.  If not, see <http://www.gnu.org/licenses/>.

"""
Parameter class and its accessory methods/functions.
"""

import sys
import types

from lava_tool.utils import to_list

# Character used to join serialized list parameters.
LIST_SERIALIZE_DELIMITER = ","


class Parameter(object):

    """A parameter with an optional dependency."""

    def __init__(self, id, value=None, depends=None):
        """Creates a new parameter.

        :param id: The name of this parameter.
        :param value: The value of this parameter. Defaults to None.
        :param depends: If this Parameter depends on another one. Defaults
                        to None.
        :type Parameter
        """
        self.id = id
        self.value = value
        self.depends = depends
        self.asked = False
        # Whether to store or not the parameter in the user config file.
        self.store = True

    def set(self, value):
        """Sets the value of the parameter.

        :param value: The value to set.
        """
        self.value = value

    def prompt(self, old_value=None):
        """Gets the parameter value from the user.

        To get user input, the builtin `raw_input` function will be used. Input
        will also be stripped of possible whitespace chars. If Enter or any
        sort of whitespace chars in typed, the old Parameter value will be
        returned.

        :param old_value: The old parameter value.
        :return The input as typed by the user, or the old value.
        """
        if not self.asked:
            if old_value is not None:
                prompt = "{0} [{1}]: ".format(self.id, old_value)
            else:
                prompt = "{0}: ".format(self.id)

            user_input = self.get_user_input(prompt)

            if user_input is not None:
                if len(user_input) == 0 and old_value:
                    # Keep the old value when user press enter or another
                    # whitespace char.
                    self.value = old_value
                else:
                    self.value = user_input

            self.asked = True

        return self.value

    @classmethod
    def get_user_input(cls, prompt=""):
        """Asks the user for input data.

        :param prompt: The prompt that should be given to the user.
        :return A string with what the user typed.
        """
        data = None
        try:
            data = raw_input(prompt).strip()
        except EOFError:
            # Force to return None.
            data = None
        except KeyboardInterrupt:
            sys.exit(-1)
        return data

    @classmethod
    def serialize(cls, value):
        """Serializes the passed value to be friendly written to file.

        Lists are serialized as a comma separated string of values.

        :param value: The value to serialize.
        :return The serialized value as string.
        """
        serialized = ""
        if isinstance(value, list):
            serialized = LIST_SERIALIZE_DELIMITER.join(
                str(x) for x in value if x)
        else:
            serialized = str(value)
        return serialized

    @classmethod
    def deserialize(cls, value):
        """Deserialize a value into a list.

        The value must have been serialized with the class instance serialize()
        method.

        :param value: The string value to be deserialized.
        :type str
        :return A list of values.
        """
        deserialized = []
        if isinstance(value, str):
            deserialized = filter(None, (x.strip() for x in value.split(
                LIST_SERIALIZE_DELIMITER)))
        else:
            deserialized = list(value)
        return deserialized


class SingleChoiceParameter(Parameter):

    """A parameter implemeting a single choice between multiple choices."""

    def __init__(self, id, choices):
        super(SingleChoiceParameter, self).__init__(id)
        self.choices = to_list(choices)

    def prompt(self, prompt, old_value=None):
        """Asks the user for their choice."""
        # Sliglty different than the other parameters: here we first present
        # the user with what the choices are about.
        print >> sys.stdout, prompt

        index = 1
        for choice in self.choices:
            print >> sys.stdout, "\t{0:d}. {1}".format(index, choice)
            index += 1

        choices_len = len(self.choices)
        while True:
            user_input = self.get_user_input("Choice: ")

            if len(user_input) == 0 and old_value:
                choice = old_value
                break
            elif user_input in [str(x) for x in range(1, choices_len + 1)]:
                choice = self.choices[int(user_input) - 1]
                break

        return choice


class ListParameter(Parameter):

    """A specialized Parameter to handle list values."""

    # This is used as a deletion character. When we have an old value and the
    # user enters this char, it sort of deletes the value.
    DELETE_CHAR = "-"

    def __init__(self, id, value=None, depends=None):
        super(ListParameter, self).__init__(id, depends=depends)
        self.value = []
        if value:
            self.set(value)

    def set(self, value):
        """Sets the value of the parameter.

        :param value: The value to set.
        """
        self.value = to_list(value)

    def add(self, value):
        """Adds a new value to the list of values of this parameter.

        :param value: The value to add.
        """
        if isinstance(value, list):
            self.value.extend(value)
        else:
            self.value.append(value)

    def prompt(self, old_value=None):
        """Gets the parameter in a list form.

        To exit the input procedure it is necessary to insert an empty line.

        :return The list of values.
        """

        if not self.asked:
            if old_value is not None:
                # We might get the old value read from file via ConfigParser,
                # and usually it comes in string format.
                old_value = self.deserialize(old_value)

            print >> sys.stdout, "Values for '{0}': ".format(self.id)

            index = 1
            while True:
                user_input = None
                if old_value is not None and (0 < len(old_value) >= index):
                    prompt = "{0:>3d}.\n\told: {1}\n\tnew: ".format(
                        index, old_value[index - 1])
                    user_input = self.get_user_input(prompt)
                else:
                    prompt = "{0:>3d}. ".format(index)
                    user_input = self.get_user_input(prompt)

                if user_input is not None:
                    # The user has pressed Enter.
                    if len(user_input) == 0:
                        if old_value is not None and \
                                (0 < len(old_value) >= index):
                            user_input = old_value[index - 1]
                        else:
                            break

                    if len(user_input) == 1 and user_input == \
                            self.DELETE_CHAR and (0 < len(old_value) >= index):
                        # We have an old value, user presses the DELETE_CHAR
                        # and we do not store anything. This is done to delete
                        # an old entry.
                        pass
                    else:
                        self.value.append(user_input)
                    index += 1

            self.asked = True

        return self.value



# Copyright (C) 2011  Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
# Author: Michael Hudson-Doyle <michael.hudson@linaro.org>
#
# This file is part of lava-dashboard-tool.
#
# lava-dashboard-tool is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# lava-dashboard-tool is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with lava-dashboard-tool.  If not, see <http://www.gnu.org/licenses/>.


from lava_tool.dispatcher import LavaDispatcher, run_with_dispatcher_class


class LaunchControlDispatcher(LavaDispatcher):

    toolname = 'lava_dashboard_tool'
    description = """
    Command line tool for interacting with Launch Control
    """
    epilog = """
    Please report all bugs using the Launchpad bug tracker:
    http://bugs.launchpad.net/lava-dashboard-tool/+filebug
    """


def main():
    run_with_dispatcher_class(LaunchControlDispatcher)
